/*
    Copyright (C) 2002, Steve Lhomme <steve.lhomme @ free.fr>
    This file is part of mpa2wav.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/**
    \file
*/

#include <cassert>
#include "mpg_wav.h"

START_M2W_NAMESPACE

void MPG_WAV_file::WriteHead(const uint32 data_size, const uint32 sample_number, bool seek_to_start)
{
    assert(myFormat);

    setAlignment(1);
    setBitsPerSample(0);
    setFormatTag(myFormat->FormatTag());
    myFormat->setExtraSize(myFormat->FormatTagSize());

    WAV_file::WriteHead(myFormat->FormatTagSize() + 4, data_size, seek_to_start);

    myFormat->Write(myFile);

    WriteSampleNumber(sample_number);

    WriteDataStart(data_size);
}

bool MPG_WAV_file::Feed(const MPA_Frame & frame)
{
    if (bFirstFrame)
    {
	setLayer(frame.Layer());
	setChannels(frame.Channels());
	setSamplingRate(frame.SamplingRate());
	myBitrateTable.insert(frame.BitRate());

	myFormat->setChannelMode(frame.ChannelMode());
	myFormat->setEmphasis(frame.Emphasis());
	myFormat->setFlags(frame.Private(),
	                   frame.Copyright(),
	                   frame.Original(),
	                   frame.Protected());
//    output.setByterate(AverageByterate);
//    output.setSampleSize(SampleSize);
	bFirstFrame = false;
    }
    else
    {
	myBitrateTable.insert(frame.BitRate());
	return Test(frame);
    }

    return true;
}

bool MPG_WAV_file::Test(const MPA_Frame & frame)
{
    if (frame.Layer() != myFormat->Layer())
	return false;

    return true;
}

uint32 MPG_WAV_file::AverageBitRate() const
{
    std::set<uint16>::const_iterator bitrate = myBitrateTable.begin();

    if (IsCBR())
	return (*bitrate * 1000);

    else return 0; /** \todo change this to the real value

    uint32 total = 0;
    uint32 count = 0;

    for(; bitrate != myBitrateTable.end();
	bitrate++)
    {
	total += *bitrate * myBitrateTable.count(*bitrate);
	count += myBitrateTable.count(*bitrate);
    }

    return (total / count);*/
}

END_M2W_NAMESPACE
