/*
    Copyright (C) 2002, Steve Lhomme <steve.lhomme @ free.fr>
    This file is part of mpa2wav.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/**
    \file
*/

#ifndef _M2W_MPG_WAV_H_
#define _M2W_MPG_WAV_H_

#ifdef _MSC_VER
#pragma warning(disable:4786)
#endif // _MSC_VER

#include <set>
#include "wav.h"
#include "mpg_format.h"
#include "mpaframe.h" // for Mpeg_Layer types

START_M2W_NAMESPACE

class MPG_WAV_file : public WAV_file {
    public:
	MPG_WAV_file(const std::string & aFileName)
	    :WAV_file(aFileName)
	    ,myFormat(NULL)
	    ,bFirstFrame(true)
	{}

	~MPG_WAV_file()
	{
	    if (myFormat)
		delete myFormat;
	}

	inline FORMAT_chunk & FormatEx() const {return *static_cast<FORMAT_chunk*>(myFormat);}

	void setSampleSize(const uint16 size) {myFormat->setSampleSize(size);}
	virtual void WriteHead(const uint32 data_size, const uint32 sample_number, bool seek_to_start);
	bool Feed(const MPA_Frame & frame);
	bool Test(const MPA_Frame & frame);
	void FixStats()
	{
	    myFormat->setBitrate(AverageBitRate(), IsCBR());
	}

	uint16 SamplingRate() const {return FormatEx().SamplingRate(); }

	bool IsCBR() const {return (myBitrateTable.size() == 1);}
	/**
	    \note in bits/sec
	*/
	uint32 AverageBitRate() const;

    protected:
	MPG_FORMAT_chunk * myFormat;
	bool               bFirstFrame;

	void setLayer(const Mpeg_Layer layer)
	{
	    if (myFormat)
		delete myFormat;
	    
	    if (layer == MPEG_LAYER_3)
		myFormat = new MP3_FORMAT_chunk;
	    else
		myFormat = new MPEG1_FORMAT_chunk;

	    myFormat->setLayer(layer);
	}

	std::set<uint16> myBitrateTable; // bitrate/frame count
};

END_M2W_NAMESPACE

#endif // _M2W_MPG_WAV_H_
